MODULE Display;
(* =========================================================================
   RISC5 Oberon LCD Text Display Functions   

   Description:
     Write line-oriented output to an LCD display

   (c) 2011-2019 CFB Software   
   http://www.astrobe.com  

  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)

IMPORT LCD := ST7735, Put;

CONST
  LF = 0AX;
  Black*   = 0;
  Red*     = 1;
  Green*   = 2;
  Blue*    = 3;
  White*   = 4;
  Cyan*    = 5;
  Magenta* = 6;
  Yellow*  = 7;
  
TYPE
  ColourMap = ARRAY 8 OF INTEGER;
  CharType = RECORD colour: BYTE; value: CHAR END;
  Row = ARRAY LCD.MaxCol OF CharType;
  Screen = ARRAY LCD.MaxRow OF Row;

VAR
  rowNo, colNo: INTEGER;
  screen: Screen;
  blankRow: Row;
  colour: INTEGER;
  colours: ColourMap;

PROCEDURE SetColour*(c: INTEGER);
BEGIN
  ASSERT((colour >= Black) & (colour <= Yellow));
  colour := c
END SetColour;
 

PROCEDURE WriteChar(ch: CharType; r, c: INTEGER);
BEGIN
  LCD.DrawChar(colours[ch.colour], ch.value, c * 8, r * 10 + 2)
END WriteChar;


PROCEDURE WriteRow(row: Row);
VAR
  c: INTEGER;
BEGIN
  FOR c := 0 TO LCD.MaxCol - 1 DO
    WriteChar(row[c], rowNo, c)
  END
END WriteRow;


PROCEDURE ScrollUp;
VAR
  r: INTEGER;
BEGIN
  (* Move rows up *)
  FOR r := 1 TO LCD.MaxRow - 1 DO
    rowNo := r - 1;
    screen[rowNo] := screen[r];
    WriteRow(screen[rowNo])
  END;
  rowNo := LCD.MaxRow - 1;
  colNo := 0;
  screen[rowNo] := blankRow
END ScrollUp;


PROCEDURE Ln*();
BEGIN
  IF rowNo < LCD.MaxRow - 1 THEN
    WriteRow(screen[rowNo]);
    INC(rowNo);
    colNo := 0;
    screen[rowNo] := blankRow
  ELSE
    ScrollUp
  END
END Ln;


PROCEDURE Char*(c: CHAR);
VAR
  ch: CharType;
BEGIN
  IF c = LF THEN
    Ln
  ELSIF c >= " " THEN
    ch.colour := colour;
    ch.value := c;
    screen[rowNo, colNo] := ch;
    INC(colNo);
    IF colNo = LCD.MaxCol THEN Ln END
  END
END Char;


PROCEDURE String*(s: ARRAY OF CHAR);
VAR
  i: INTEGER;
BEGIN
  i := 0;
  WHILE s[i] # 0X DO Char(s[i]); INC(i) END
END String;


PROCEDURE Int*(n: INTEGER);
VAR 
  s: ARRAY 10 OF CHAR;
  sLen: INTEGER;
BEGIN
  Put.Init;
  Put.Int(n);
  Put.EOS(s);
  String(s)
END Int;


PROCEDURE Clear*();
VAR
  ch: CharType;
BEGIN
  (* Initialise the memory representation of the screen *)
  ch.colour := Black;
  ch.value := " ";
  FOR colNo := 0 TO LCD.MaxCol - 1 DO blankRow[colNo] := ch END;
  FOR rowNo := 0 TO LCD.MaxRow - 1 DO screen[rowNo] := blankRow END;
  rowNo := 0;
  colNo := 0;
  LCD.ClearScreen(LCD.Black)
END Clear;


PROCEDURE Init*();
BEGIN
  LCD.Init();
  Clear();
  SetColour(White)
END Init;


BEGIN
  colours[Black]   := LCD.Black;
  colours[Red]     := LCD.Red;
  colours[Green]   := LCD.Green;
  colours[Blue]    := LCD.Blue;
  colours[White]   := LCD.White;
  colours[Cyan]    := LCD.Cyan;
  colours[Magenta] := LCD.Magenta;
  colours[Yellow]  := LCD.Yellow
END Display.
