MODULE ST7735;
(* =========================================================================  
   Example RISC5 Oberon Module  
 
   Description:
     Sitronix ST7735 Display Controller Driver for 128x160 TFT display

   Target:
     RISC5 systems
     
   Tested on: 
     Digilent Arty A7/S7 + AdaFruit 1.8" TFT Display
   
   Reference:
     Sitronix ST7735 262K Color Single-Chip TFT Controller/Driver
     v2.1 2010-02-01
     
   (c) 2015-2019 CFB Software
   http://www.astrobe.com

  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)

IMPORT Font, GPIO, SPI, Timer;

CONST 
  SLPOUT  = 011H; (* Sleep out & booster on *)
  CASET   = 02AH; (* Column address set *)
  RASET   = 02BH; (* Row address set *)
  RAMWR   = 02CH; (* Writing to memory *)
  MADCTL  = 036H; (* Memory data access control *)
  COLMOD  = 03AH; (* Colour mode - interface pixel format *)
  DISPON  = 029H; (* Display on *)
  
  (* 16-bit RRRRR-GGGGGG-BBBBB color definitions *)
  Black*   = 00000H;
  Red*     = 0F800H;
  Green*   = 007E0H;
  Blue*    = 0001FH;
  White*   = Red + Green + Blue;
  Cyan*    = Green + Blue;
  Magenta* = Red + Blue;
  Yellow*  = Red + Green;
  
  MaxX* = 159;
  MaxY* = 127;
 
  (* 8 x 10 font *)
  MaxCol* = MaxX DIV 8;
  MaxRow* = MaxY DIV 10;

  (* GPIO Outputs *)
  BackLight = 5; (* IO5 *)
  Reset     = 6; (* IO6 *)
  A0        = 8; (* IO8 *)

  SPINone = {};
  SPI2    = {2};
  SPIFast = {3};
  SPIData = -48; 
  SPICtrl = -44;

VAR
  gpio: SET;
  
  (* ========================================================================= *) 
  PROCEDURE SendCommand(data: BYTE);
  BEGIN
    GPIO.Reset(A0);
    SPI.Select(SPI2);
    SPI.SendByte(data);
    GPIO.Set(A0)
  END SendCommand;

  (* ========================================================================= *) 
  PROCEDURE SendWord(data: INTEGER);
  VAR
    b: ARRAY 4 OF BYTE;
  BEGIN
    SPI.Select(SPI2 + SPIFast);
    b[0] := ASR(data, 24);
    b[1] := ASR(data, 16);
    b[2] := ASR(data, 8);
    b[3] := data;
    SPI.SendWord(b)
  END SendWord;

  (* ========================================================================= *) 
  PROCEDURE SetXY(x, y: INTEGER);
  BEGIN
    (* Column address *)
    SendCommand(CASET);
    SendWord(LSL(y, 16) + y);
    SendCommand(RASET); 
    SendWord(LSL(x, 16) + x);
    (* Subsequent data will be written to the display *)
    SendCommand(RAMWR) 
  END SetXY;
  
  (* ========================================================================= *) 
  PROCEDURE SetY(y1, y2: INTEGER);
  BEGIN
    SendCommand(CASET);
    SPI.SendByte(0);
    SPI.SendByte(y1);
    SPI.SendByte(0);
    SPI.SendByte(y2);
  END SetY;
  
  (* ========================================================================= *) 
  PROCEDURE SetX(x1, x2: INTEGER);
  BEGIN
    SendCommand(RASET);
    SPI.SendByte(0);
    SPI.SendByte(x1);
    SPI.SendByte(0);
    SPI.SendByte(x2);
  END SetX;
  
 (* ************************************************************************************* *)
 (* Draws a dot in the specified color at the specified x and y coordinates *)
 (* color = 16-bit color value rrrrrggggggbbbbb *)
  PROCEDURE DrawDot*(colour, x, y: INTEGER);
  BEGIN
    IF (x <= MaxX) & (y <= MaxY) & (x >= 0) & (y >= 0) THEN 
      SPI.Select(SPI2); 
      (* Column address *)
      SetXY(x, y);
      SPI.SendByte(colour DIV 256);
      SPI.SendByte(colour MOD 256);
      SPI.Select(SPINone) 
    END 
  END DrawDot;

  (* ========================================================================= *) 
  PROCEDURE DrawRectangle*(colour, x, y, width, length: INTEGER);
  VAR
    i, count, colourWord: INTEGER;
  BEGIN
    SPI.Select(SPI2); 
    SendCommand(CASET); 
    SendWord(LSL(y, 16) + y + length - 1); 
    SendCommand(RASET); 
    SendWord(LSL(x, 16) + x + width - 1); 
    SendCommand(RAMWR);
    count := (width * length) DIV 2;
    colourWord := LSL(colour, 16) + colour;
    FOR i := 0 TO count - 1 DO 
      SendWord(colourWord)
    END; 
    SPI.Select(SPINone) 
  END DrawRectangle;

  (* ========================================================================= *) 
  PROCEDURE DrawChar*(colour: INTEGER; ch: CHAR; x1, y1: INTEGER);
  VAR
    i: INTEGER;
    x2, y2: INTEGER;
    bmp, bmp1, bmp2: INTEGER;
  BEGIN
    Font.CharToBitmap(ch, bmp1, bmp2);
    x2 := x1 + Font.Width - 1;
    y2 := y1 + Font.Height - 1;
    ASSERT((x2 <= MaxX) & (y2 <= MaxY));
    SetY(y1, y2);
    SetX(x1, x2);
    SendCommand(RAMWR);
    bmp := bmp1;
    FOR i := 0 TO Font.Width * Font.Height - 1 DO 
      IF i = 32 THEN bmp := LSL(bmp2, 16) END;
      IF bmp < 0 THEN 
        SPI.SendByte(BFX(colour, 15, 8));
        SPI.SendByte(BFX(colour,  7, 0))
      ELSE 
        SPI.SendByte(0);
        SPI.SendByte(0)
      END;
      bmp := LSL(bmp, 1);
    END;
  END DrawChar;
  
  (* ========================================================================= *) 
  PROCEDURE ClearScreen*(colour: INTEGER);
  BEGIN
    DrawRectangle(colour, 0, 0, MaxX + 1, MaxY + 1)
  END ClearScreen;
  
  (* ========================================================================= *) 
  (* Initialise the Sirtronix ST7735 TFT Controller *)
  PROCEDURE Init*();
  VAR
    ok: BOOLEAN;
  BEGIN
    ok := Font.Load("Font.res");
    ASSERT(ok);
    (* Set GIO pins as outputs *)
    GPIO.SetOutputs({A0, BackLight, Reset});
    GPIO.Reset(Reset);
    GPIO.Set(Reset);
    Timer.MSecDelay(120); 
    SPI.Select(SPI2); 
    SendCommand(SLPOUT); 
    Timer.MSecDelay(120); 
    (* RGB16 5-6-5 *)
    SendCommand(COLMOD); 
    SPI.SendByte(05H); 
    (* Portrait RGB *)
    SendCommand(MADCTL);
    SPI.SendByte(046H); 
    SendCommand(DISPON);
    SPI.Select(SPINone); 
    GPIO.Set(BackLight)
  END Init;
  
END ST7735.
